use crate::data::record::{Field, FieldRecord};
use std::fmt;
use uuid::Uuid;

#[derive(Clone)]
pub struct ID {
    data: Uuid,
}

impl ID {
    pub fn new() -> Self {
        Self {
            data: Uuid::new_v4(),
        }
    }
}

impl FieldRecord for ID {
    fn new_field() -> Field {
        Field::ID(ID::new())
    }

    fn get_type() -> String {
        "id".to_string()
    }
}

impl fmt::Display for ID {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{}", self.data)
    }
}

#[cfg(test)]
mod fielddata {
    use super::*;

    #[test]
    fn new_ids_are_unique() {
        let mut ids: Vec<String> = Vec::new();
        for _ in 1..10 {
            match ID::new_field() {
                Field::ID(id) => {
                    let id_string = id.to_string();
                    assert!(!ids.contains(&id_string), "'{}' repeated", id_string);
                    ids.push(id_string);
                }
            }
        }
    }

    #[test]
    fn get_type() {
        assert_eq!(ID::get_type(), "id");
    }
}

#[cfg(test)]
mod id {
    use super::*;

    #[test]
    fn id_are_unique() {
        let mut ids: Vec<String> = Vec::new();
        for _ in 1..10 {
            let id = ID::new();
            let id_string = id.to_string();
            assert!(!ids.contains(&id_string), "'{}' repeated", id_string);
            ids.push(id_string);
        }
    }
}
